function [f_a, a_a, d_a, f_off] = ss_alias(f, a, d, f_off, fs, sym)
% SS_ALIAS - Alias filter specifications into effective bandwidth
%   
% function [f_a, a_a, d_a, f_off] = ss_alias(f, a, d, f_off, fs, sym)
%
% Inputs:     
% f - frequency band edges, each band monotonically increasing, 
%     length(f) = 2 * number of bands (Hz)
% a - amplitude for each band [0..1]
% d - ripple weighting for each band 
% f_off - offset frequency in Hz (if [], then will be 
%         set to include as many passbands within one sampling
%         interval as possible    
% fs - sampling frequency (Hz)
% sym - flag indicating whether frequency response should be symmetric    
%
% Outputs: 
% f_a - Aliased frequency band edges into normalized freq [-1..1]
% a_a - amplitude of aliased bands
% d_a - ripple weighting of aliased bands
% f_off - offset frequency in Hz
%     
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Spectral-Spatial RF Pulse Design for MRI and MRSI MATLAB Package
%
% Authors: Adam B. Kerr and Peder E. Z. Larson
%
% (c)2007-2011 Board of Trustees, Leland Stanford Junior University and
%	The Regents of the University of California. 
% All Rights Reserved.
%
% Please see the Copyright_Information and README files included with this
% package.  All works derived from this package must be properly cited.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% $Header: /home/adam/cvsroot/src/ss/ss_alias.m,v 1.8 2012/02/01 00:41:23 peder Exp $
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% $Log: ss_alias.m,v $
% Revision 1.8  2012/02/01 00:41:23  peder
% Added standard copyright notice to all functions
%
% Revision 1.7  2010/03/19 20:35:39  peder
% Very minor edits to comments
%
% Revision 1.6  2007-10-29 15:38:33  adam
% Fix determination of offset frequency so that largest number
% of bands are within one sampling bandwidth region
%
% Revision 1.5  2007-09-13 16:50:52  adam
% Fix bug in ss_alias that allowed incompatible overlaps through.
%
% Revision 1.4  2007-09-09 18:51:38  adam
% Add check for number of ramp samples going to 1.
%
% Revision 1.3  2007-09-07 17:26:47  adam
% Integrating ss_spect_correct, as well as EP support, almost(!!) working
%
% Revision 1.2  2007-09-05 20:59:24  adam
% Add water/fat examples
% First pass at adding symmetric filter design---still needs some work
% Get rid of SLR trial in EPI spectral correct routine---will fix later
%
% Revision 1.1.1.1  2007-08-29 15:58:20  adam
% Starting sources
%
% Revision 1.1.1.1  2007-08-29 15:45:32  adam
% Starting sources
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % check input parameters
    %
    if nargin < 6, 
	error(['Usage: [f_a, a_a, d_a, f_off] = ss_alias(f, a, d,' ...
	       ' f_off, fs, sym)\n']);
	
    end;
    
    if mod(length(f), 2) ~= 0, 
	error(['Frequency vector (f) of band pairs must be even' ...
	       ' length']);
    end;
    nf = length(f)/2;
    if length(d) ~= nf, 
	error('Ripple vector incorrect length');
    end;
    if length(a) ~= nf, 
	error('Amplitude vector incorrect length');
    end;
    
    df = diff(f);
    if (min(df(1:2:end)) <= 0) 
	error('Frequency bands must be monotonic increasing');
    end;

    if (max(df(1:2:end)) >= fs)
	f_a = [];
	a_a = [];
	d_a = [];
	f_off = 0;
	return;
    end;
    
    % If sym flag then either set or check f_off
    %
    if (sym), 
	if ~isempty(f_off), 
	    % Check specified frequency offset 
	    % Make sure it is less than midpoint 
	    % of first band or > than midpoint of 
	    % top band
	    %
	    if ((f_off > (f(2)-f(1))/2) && ...
		(f_off < (f(end) - f(end-1))/2) )
		error('f_off not at band edges');
	    end;
	else
	    % Set f_off to left band edge
	    %
	    f_off = (f(2)+f(1))/2;
	end;
    else
	% Set f_off to include as many bands 
	% as possible in one sampling interval
	if isempty(f_off)
	    if (max(f) - min(f)) <= fs, 
		f_off = (max(f) + min(f))/2;
	    else
		% Find how many bands are in -fs/2..fs/2
		%
		f_l = f(1:2:end);
		f_u = f(2:2:end);
		f_off_test = linspace(min(f)+fs/2, max(f)-fs/2, 50);
		for off_idx = 1:length(f_off_test), 
		    f_off = f_off_test(off_idx);
		    band_in(off_idx,:) = ((f_l >= f_off-fs/2) & (f_l <= f_off+fs/2) & ...
					  (f_u >= f_off-fs/2) & (f_u <= f_off+fs/2));
		    nband(off_idx) = sum(band_in(off_idx,:));
		end;
		[nb, off_idx] = max(nband);
		in_idx = find(band_in(off_idx,:));
		f_off = (f(in_idx(1)*2-1) + f(in_idx(end)*2))/2;
	    end;
	end;
    end;
	
    % If symmetric frequency response, then construct 
    % mirrored response, update number of bands
    if (sym)
	f_h = f - f_off;
	f = [-f_h(end:-1:1) f_h] + f_off; 
	a = [a(end:-1:1) a];
	d = [d(end:-1:1) d];
	nf = length(f)/2;
    end;

    % Go through each band pair, determine aliased frequencies 
    % in normalized space, and split if necessary
    %
    new_idx = 1;
    for idx = 1:nf, 
	% Normalize frequencies
	%
	fnorm(idx*2-1) = (f(idx*2-1) - f_off) / (fs/2); 
	fnorm(idx*2) = (f(idx*2) - f_off) / (fs/2); 

	% Get aliased frequencies
	%
	fa1 = mod(fnorm(idx*2-1)+1,2)-1;
	fa2 = mod(fnorm(idx*2)+1,2)-1;
	
	% Check to see if endpoints need to be 
	% shifted
	if (fa2 < fa1) && (fa2 == -1)
	    fa2 = 1;
	end;
	if (fa2 < fa1) && (fa1 == 1)
	    fa1 = -1;
	end;
	
	% Check to see if new band needs to be created
	%
	if (fa2 < fa1), 
	    f_a(new_idx*2-1) = -1;
	    f_a(new_idx*2) = fa2;
	    a_a(new_idx) = a(idx);
	    d_a(new_idx) = d(idx);
	    new_idx = new_idx + 1;

	    f_a(new_idx*2-1) = fa1;
	    f_a(new_idx*2) = 1;
	    a_a(new_idx) = a(idx);
	    d_a(new_idx) = d(idx);
	    new_idx = new_idx + 1;
	else
	    f_a(new_idx*2-1) = fa1;
	    f_a(new_idx*2) = fa2;
	    a_a(new_idx) = a(idx);
	    d_a(new_idx) = d(idx);
	    new_idx = new_idx + 1;
	end;
    end;

    % Check for overlaps
    %   -- a little complicated
    overlap_found = 1;
    incompatible_overlap = 0;
    while (overlap_found && ~incompatible_overlap) 
	% Check each frequency band for overlap
	%
	nf = length(f_a)/2;
	f_tmp = [];
	a_tmp = [];
	d_tmp = [];
	for chk_idx = 1:nf, 
	    % Copy frequency, ripple, amplitude
	    %
	    f_tmp(chk_idx*2-1) = f_a(chk_idx*2-1);
	    f_tmp(chk_idx*2) = f_a(chk_idx*2);
	    d_tmp(chk_idx) = d_a(chk_idx);
	    a_tmp(chk_idx) = a_a(chk_idx);

	    % Check current band edges with all remaining
	    % bands for overlap
	    fl_1 = f_a(chk_idx*2-1);
	    fu_1 = f_a(chk_idx*2);
	    for rem_idx = chk_idx+1:nf, 
		% Initialize flags
		%
		overlap_ok = 0;
		overlap_found = 0;
	    
		% Check to see if overlap would be compatible
		%    i.e. same ripple, same amplitude
		if ((d_a(chk_idx) == d_a(rem_idx)) && ...
		    (a_a(chk_idx) == a_a(rem_idx))), 
		    overlap_ok = 1;
		end;
		
		% Check if band edges overlap
		%
		fl_2 = f_a(rem_idx*2-1);
		fu_2 = f_a(rem_idx*2);
		if (fl_1 <= fl_2) && (fu_1 >= fl_2)
		    % Fix overlapping band
		    %
		    f_tmp(chk_idx*2-1) = fl_1;
		    f_tmp(chk_idx*2) = max(fu_1, fu_2);

		    % Copy other bands
		    %
		    for cp_idx = (chk_idx+1):nf, 
			if cp_idx ~= rem_idx, 
			    f_tmp = [f_tmp f_a(cp_idx*2-1) f_a(cp_idx*2)];
			    a_tmp = [a_tmp a_a(cp_idx)];
			    d_tmp = [d_tmp d_a(cp_idx)];
			end;
		    end;
		    overlap_found = 1;
		    break;		% Start over
		elseif (fu_1 >= fu_2) && (fl_1 <= fu_2)
		    % Fix overlapping bands
		    %
		    f_tmp(chk_idx*2-1) = min(fl_1, fl_2);
		    f_tmp(chk_idx*2) = fu_1;

		    % Copy other bands
		    %
		    for cp_idx = (chk_idx+1):nf, 
			if cp_idx ~= rem_idx, 
			    f_tmp = [f_tmp f_a(cp_idx*2-1) f_a(cp_idx*2)];
			    a_tmp = [a_tmp a_a(cp_idx)];
			    d_tmp = [d_tmp d_a(cp_idx)];
			end;
		    end;
		    overlap_found = 1;
		    break;		% Start over
		elseif (fl_1 >= fl_2) && (fu_1 <= fu_2)
		    % Fix overlapping bands
		    %
		    f_tmp(chk_idx*2-1) = fl_2;
		    f_tmp(chk_idx*2) = fu_2;

		    % Copy other bands
		    %
		    for cp_idx = (chk_idx+1):nf, 
			if cp_idx ~= rem_idx, 
			    f_tmp = [f_tmp f_a(cp_idx*2-1) f_a(cp_idx*2)];
			    a_tmp = [a_tmp a_a(cp_idx)];
			    d_tmp = [d_tmp d_a(cp_idx)];
			end;
		    end;
		    overlap_found = 1;
		    break;		% Start over
		end;
	    end;
	    if overlap_found, 
		incompatible_overlap = ~overlap_ok; 
		f_a = f_tmp;
		d_a = d_tmp;
		a_a = a_tmp;
		break;			% Start over from beginning
	    end;
	end;
    end;
    
    % If incompatible overlap, then return empty matrices
    %
    if incompatible_overlap, 
	f_a = []; 
	d_a = [];
	a_a = [];
	return;
    end;
    
    % Sort frequencies into ascending order
    %
    [f_a, idx_sort] = sort(f_a);
    a_a = a_a(idx_sort(2:2:end)/2);
    d_a = d_a(idx_sort(2:2:end)/2);

    % If symmetric response then return only positive bands
    %
    if (sym)
	% Find right band edges that are positive, and fix 
	% return variables
	%
	idx_pos = find(f_a(2:2:end) > 0);
	f_tmp = [];
	for idx = 1:length(idx_pos), 
	    f_tmp(idx*2-1) = f_a(idx_pos(idx)*2-1);
	    f_tmp(idx*2) = f_a(idx_pos(idx)*2);
	end;
	f_a = max(0,f_tmp);
	a_a = a_a(idx_pos);
	d_a = d_a(idx_pos);
    end;
    
    return;	
	
    
    

