function [g, rf, fs_best, z_plot, f_plot, m_plot] = ...
    ss_design(z_thk, z_tb, z_de, f, a_angs, de,... 
	      ptype, z_ftype, s_ftype, ss_type, ...
	      f_off, dbg);
    
					
% SS_DESIGN - Design spectral-spatial pulse
%   
% [g, rf, fs_best, z_plot, f_plot, m_plot] = ...
%     ss_design(z_thk, z_tb, z_de, f, a_angs, de,...
%               ptype, z_ftype, s_ftype, ss_type, ...
%               f_off, dbg)
%
% INPUTS
%   z_thk - slice thickness (cm)
%   z_tb - spatial time-bandwidth
%   z_de - spatial ripples, [pass_ripple, stop_ripple]
%   f - spectral band edge specification (Hz)
%   a_ang - spectral band flip angle specification (radians)
%   de - spectral band ripples
%   ptype - spatial pulse type: 'ex' (default), 'se', 'sat', 'inv'
%   z_ftype - spatial filter type: 'ms', 'ls', 'pm' (default), 'min', 'max'
%   s_ftype - spectral filter type: 'min' (default), 'max', 'lin'
%   ss_type - spectral-spatial type: 'Flyback Whole' (default),
%       'Flyback Half', 'EP Whole', 'EP Half',
%       'EP Whole Opp-Null', 'EP Half Opp-Null'
%   f_off - center frequency (empty to let ss_design choose)
%   dbg - print debug messages: 0-none (default), 1-some, 2-more
%
% OUTPUTS
%   g - gradient (G/cm)
%   rf - RF (G)
%   fs_best - spectral sampling frequency (Hz)
%   z_plot, f_plot, m_plot - ss_plot() outputs, ranges and values in figure
%
% See scripts in examples/ folder demonstrations of how to use this
% function.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Spectral-Spatial RF Pulse Design for MRI and MRSI MATLAB Package
%
% Authors: Adam B. Kerr and Peder E. Z. Larson
%
% (c)2007-2011 Board of Trustees, Leland Stanford Junior University and
%	The Regents of the University of California. 
% All Rights Reserved.
%
% Please see the Copyright_Information and README files included with this
% package.  All works derived from this package must be properly cited.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% $Header: /home/adam/cvsroot/src/ss/ss_design.m,v 1.30 2012/02/01 00:41:23 peder Exp $
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% $Log: ss_design.m,v $
% Revision 1.30  2012/02/01 00:41:23  peder
% Added standard copyright notice to all functions
%
% Revision 1.29  2012/01/19 00:07:18  peder
% In case of design failure, ss_design increases the ripples in all bands to find offending bands, and then returns a pulse with increased ripple than what was specified.  Added example of this to demo.m
%
% Revision 1.28  2012/01/11 08:19:23  peder
% Added power measurement to ss_design output
%
% Revision 1.27  2012/01/11 06:59:27  peder
% New disclaimers, copyrights, and references for package.  disclaimer only added to ss_design so far
%
% Revision 1.26  2012/01/06 01:04:53  peder
% Added interactive choice from all pulse solutions
%
% Revision 1.25  2011/09/27 22:06:17  peder
% examples/*.m are all now working and updated.  minor fix to fir_minphase_power to catch error.  Added many comments to functions
%
% Revision 1.24  2010-06-08 18:08:24  peder
% added error message
%
% Revision 1.23  2008-05-23 18:46:15  peder
% Added scaling for inv/se pulses with beta close to 1.  Fixed minor ss_plot bugs
%
% Revision 1.22  2008-05-23 17:31:31  peder
% Changed ss_design call to take flip angles, not filter amplitudes, so that inv/se pulses are easily prescribed
%
% Revision 1.21  2008-05-23 02:00:20  peder
% Added different pulse types to ss_plot
%
% Revision 1.20  2008-05-23 00:27:12  peder
% Updated ripple specs to work for multiband spin-echo and inversion pulses.  Working towards incorporating spin-echo and inversion into design
%
% Revision 1.19  2007-12-04 23:50:44  adam
% Change plot range for frequency in ss_plot
%
% Revision 1.18  2007-10-31 15:56:23  adam
% Fix scaling of amplitude band specfication to account for tip angles
%
% Revision 1.17  2007-10-29 15:39:04  adam
% Fix specification of beta polynomial amplitudes
%
% Revision 1.16  2007-10-25 21:05:13  adam
% Add option for linear-phase spectral filter
%
% Revision 1.15  2007-10-05 16:42:25  adam
% Fix up some printing bugs
%
% Revision 1.14  2007-09-19 03:26:54  adam
% Fix initial gradient design to get unequal lobes if desired.  Uses call to grad_ss now.
%
% Revision 1.13  2007-09-18 17:48:12  adam
% Fix up some plotting, error checking
%
% Revision 1.12  2007-09-08 02:37:29  adam
% Fixing up demo script
% Fixed up ss_plot to look a little better
%
% Revision 1.11  2007-09-07 20:59:39  adam
% Looks like everything is working now...
%
% Revision 1.10  2007-09-07 17:26:47  adam
% Integrating ss_spect_correct, as well as EP support, almost(!!) working
%
% Revision 1.9  2007-09-05 20:59:24  adam
% Add water/fat examples
% First pass at adding symmetric filter design---still needs some work
% Get rid of SLR trial in EPI spectral correct routine---will fix later
%
% Revision 1.8  2007-09-04 17:48:09  adam
% Added refocussing lobes
%
% Revision 1.7  2007-09-04 16:53:20  adam
% Got rid of call to brian's bh_verse...
% there was some strange behaviour that I didn't bother tracking down
% Also fixed bug in choosing "best" rf
%
% Revision 1.5  2007-08-31 00:00:50  adam
% Adding B1-minimization loop
%
% Revision 1.4  2007-08-30 05:21:18  adam
% Fixed up some nuisance bugs.
% Saw some strange behaviour with 'verse' and so modified it slightly
% into ss_verse.  Still not sure it is best it can be.
%
% Revision 1.3  2007-08-30 00:23:58  adam
% Adding VERSE_FRACTION capability
%
% Revision 1.1.1.1  2007-08-29 15:58:20  adam
% Starting sources
%
% Revision 1.1.1.1  2007-08-29 15:45:31  adam
% Starting sources
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Check all inputs
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   
    if (nargin < 6), 
	error(['Usage: ss_design(z_thk, z_tb, z_d, f, a_angs, d,' ...
	       'ptype, z_fttype, s_ftype, ss_type, foff)']); 
    end;
    
    % Check ptype
    %
    if (nargin < 7) || isempty(ptype), 
	ptype = 'ex'; 
    else
	switch ptype, 
	 case {'ex', 'se', 'sat', 'inv'}
	 otherwise 
	  error(sprintf(['Spatial pulse type (ptype) of: %s not' ...
			 ' recognized'], ptype));
	end;
    end;
    
    % Check z_ftype
    %
    if (nargin < 8) || isempty(z_ftype), 
	z_ftype = 'pm'; 
    else
	switch z_ftype, 
	 case {'ms', 'ls', 'pm', 'min', 'max'}
	 otherwise 
	  error(sprintf(['Spatial filter type (z_ftype) of: %s not' ...
			 ' recognized'], z_ftype));
	end;
    end;
    
    % Check s_ftype
    %
    if (nargin < 9) || isempty(s_ftype), 
	s_ftype = 'min'; 
    else
	switch s_ftype, 
	 case {'min', 'max', 'lin'}
	 otherwise 
	  error(sprintf(['Spectral filter type (s_ftype) of: %s not' ...
			 ' recognized'], s_ftype));
	end;
    end;
    
    % Check ss_type
    %
    if (nargin < 10) || isempty(ss_type), 
	ss_type = 'Flyback Whole'; 
    else
	switch ss_type, 
	 case {'Flyback Whole', 'Flyback Half', ...
	       'EP Whole', 'EP Half', 'EP Whole Opp-Null', ...
	       'EP Half Opp-Null'}
	 otherwise 
	  error(sprintf(['Spectral-spatial type (ss_type) of: %s not' ...
			 ' recognized'], ss_type));
	end;
    end;

    % Check f_off
    %
    if (nargin < 11) || isempty(f_off), 
	f_off = [];
    end;
    
    % Check dbg
    %
    if (nargin < 12) || isempty(dbg), 
	dbg = 0;
    end;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Initialize globals
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    ss_globals;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Convert "a_angs" to Beta polynomial "a"
    % Convert effective ripples to polynomial ripples 
    % depending on pulse type
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ang = max(a_angs);
    
    if SS_SLR_FLAG, 
	a = sin(a_angs/2)/sin(ang/2);
	%a = asin(sin(ang)*a)/ang;
    else
	a = a_angs/ang;
    end
    
    [d,a,ang] = rf_ripple(de, a, ang, ptype);
    z_d = rf_ripple(z_de, [1 0], ang, ptype);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Calculate gradient lobe required to achieve spatial
    % prescription
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Calculate cycles/cm required
    %
    kz_max = z_tb / z_thk;		% cycles/cm
    kz_area = kz_max / SS_GAMMA;	% G/cm * s
    
    % Note:  If ss_type is EP, then SS_EQUAL_LOBES must be 1
    %
    if strfind(ss_type, 'EP')
	SS_EQUAL_LOBES = 1;
    else;
	SS_EQUAL_LOBES = 0;
    end;

    % Get SS gradient lobes that gives this area with 
    % the desired fraction of sloped gradient included
    % (this is to limit amount of versing required)
    %
    [gpos, gneg, g1, g2, g3] = ...
	grad_ss(kz_area, [], SS_VERSE_FRAC, SS_MXG, SS_MXS, ...
		SS_TS, SS_EQUAL_LOBES);
    
    % Calculate maximum spectral sampling frequency
    %
    lobe = [gpos gneg];
    t_poslobe = length(gpos) * SS_TS;
    t_lobe = length(lobe) * SS_TS;
    fs_scale = 1;
    if strfind(ss_type, 'EP')
	fs_max = 2/t_lobe;
	if ~strfind(ss_type, 'Opp-Null')
	    fs_scale = 1/2;		% If true null, make sure 
	end;				% aliasing check is for half-frequency
    else
	fs_max = 1/t_lobe;
    end;

    % Estimate sampling frequencies that won't cause
    % incompatible aliasing of stopbands/passbands
    %
    switch ss_type, 
     case {'Flyback Half', 'EP Half', 'EP Half Opp-Null'},
      sym_flag = 1;
     otherwise
      sym_flag = 0;
    end;
      
    fdiff = diff(f);
    fwidths = sort(fdiff(1:2:end), 2, 'descend');
    fs_min = sum(fwidths(1:2))/2;		% Loose estimate on lower bound
    df = (fs_max-fs_min)/(SS_NUM_FS_TEST-1);
    for idx = 1:SS_NUM_FS_TEST,
	fs_test(idx) = fs_min + (idx-1)*df;
	% Make sure fs_cur is integer number of samples
	%
	nsamp = ceil(1/(fs_test(idx)*SS_TS));
	fs_test(idx) = 1/(nsamp*SS_TS);

	% Test aliasing of frequencies into operating BW
	%
	[f_a, a_a, d_a, fo] = ss_alias(f,a,d,f_off,fs_test(idx)*fs_scale,sym_flag);
	if (isempty(f_a)), 
	    fs_ok(idx) = 0;
	else
	    fs_ok(idx) = 1;
	end;
    end;
    if (dbg >= 2)
	figure;
	plot(fs_test, fs_ok, '*');
	title('Feasible Spectral Sampling Frequencies');
	xlabel('Frequency [Hz]');
	ylabel('Flag (1=OK, 0=No Good)');
	drawnow;
	pause(1);
    end;
    
    if (~any(fs_ok == 1)),
	fs_over = fs_test(end);
	while isempty(f_a)
	    fs_over = fs_over + df;
	    [f_a, a_a, d_a, fo] = ss_alias(f,a,d,f_off,fs_over*fs_scale,sym_flag);
	end;
	
	fprintf(1,'ss_design: Incompatible aliasing of frequency spec\n');
	fprintf(1,'at all tested frequencies. \n');
	fprintf(1,'Current max sampling is: %6.1f\n', fs_max);
	fprintf(1,'Estimated required sampling is: %6.1f\n', fs_over);
	fprintf(1, 'Try any of the following to incr:\n');
	fprintf(1,'   - Decrease spatial TBW\n');
	fprintf(1,'   - Increase slice thickness\n');
	fprintf(1,'   - Increase VERSE fraction\n');
	fprintf(1,'   - Decrease frequency band widths\n');
	switch (ss_type), 
	 case {'Flyback Whole', 'Flyback Half'}
	  fprintf(1,'   - Try EPI type SS\n');
	end;
	error('No good fs');
    end;
    fs_bands_left = find(diff([0 fs_ok]) == 1);
    fs_bands_right = find(diff([fs_ok 0]) == -1);
    fs_bands = [fs_bands_left; fs_bands_right];
    fs_bands = fs_bands(:)';

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Iterate on lobe width trying to meet B1 requirements
    % with minimum-time pulse
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    nsolutions = 0;
    
    if (SS_NUM_LOBE_ITERS == 1), 
	fs_top = fs_bands(end);
	fs_best = fs_test(fs_top);

	% Call design script for SS that calculates rf, g that meets 
	% sampling requirements 
	%

	if strfind(ss_type, 'Flyback')
	    [rf, g] = ss_flyback(ang, z_thk, z_tb, z_d, f, a, d, fs_best, ...
					   ptype, z_ftype, s_ftype, ss_type, ...
					   f_off, dbg);
	else
	    [rf, g] = ss_ep(ang, z_thk, z_tb, z_d, f, a, d, fs_best, ...
				      ptype, z_ftype, s_ftype, ss_type, ...
				      f_off, dbg);
	end;
	    
	
	if ~isempty(rf),
	    b1_best = max(abs(rf));
	    dur_best = length(rf) * SS_TS;
        pow_best = sum(abs(rf).^2) * SS_TS;
        nsolutions = 1;
    	fprintf(1,'Solution(s) exists!\n');
    	fprintf(1, 'Fs: %6.1f B1: %5.3fG Power: %5.3e G^2 ms Dur: %4.1fms\n', fs_best, b1_best, pow_best, dur_best*1e3); 
    else
        fprintf(1, 'Fs: %6.1f *** No Soln ***\n', fs_best);

	end;
    else
	fprintf(1, 'Iterating on spectral sampling frequency to reduce B1\n');

	% Keep iterating on pulse design until B1 requirement is 
	% met with highest sampling rate possible
	%
	dur_best = inf;
	b1_best = inf;
	nbands = length(fs_bands)/2;
	for band = nbands:-1:1,
	    % Try each band
	    %
	    fs_bot = fs_bands(band*2-1);
	    fs_top = fs_bands(band*2);
	    d_idx = floor((fs_top - fs_bot + 1)/(SS_NUM_LOBE_ITERS-1));
	    d_idx = max(1,d_idx);
	    niter = ceil((fs_top-fs_bot+1)/d_idx);
	    for iter = niter:-1:1,
		idx = fs_bot + (iter-1)*d_idx;
		if iter == niter, 
		    idx = fs_top;
		end;
		fs = fs_test(idx);

		% Call design script for SS that calculates rf, g that meets 
		% sampling requirements 
		%
		if strfind(ss_type, 'Flyback')
		    [rf, g] = ss_flyback(ang, z_thk, z_tb, z_d, f, a, d, fs, ...
					 ptype, z_ftype, s_ftype, ss_type, ...
					 f_off, dbg);
		else
		    [rf, g] = ss_ep(ang, z_thk, z_tb, z_d, f, a, d, fs, ...
				    ptype, z_ftype, s_ftype, ss_type, ...
				    f_off, dbg);
		end;
		    
		if isempty(rf), 
		    fprintf(1, 'Band: %d/%d Iter: %d/%d Fs: %6.1f *** No Soln ***\n', ...
			    nbands-band+1, nbands, niter-iter+1, niter, fs);
		    continue;
		end;
		
        nsolutions = nsolutions+1;
        
        rfall{nsolutions} = rf;
        gall{nsolutions} = g;
        fsall(nsolutions) = fs;
        
		b1 = max(abs(rf));
		dur = length(rf) * SS_TS;
        pow = sum(abs(rf).^2) * SS_TS;
        
        infoall{nsolutions} = sprintf('Fs: %6.1f B1: %5.3fG Power: %5.3e G^2 ms Dur: %4.1fms', ...
			fs, b1, pow, dur*1e3);
		fprintf(1, 'Band: %d/%d Iter: %d/%d %s\n', nbands-band+1, nbands, niter-iter+1, niter, infoall{nsolutions}); 
        
        
         if ((b1 <= SS_MAX_B1) && (dur < dur_best))
 		    b1_best = b1;
 		    dur_best = dur;

            Ibest = nsolutions; 		
 		elseif ((b1_best > SS_MAX_B1) && (b1 < b1_best))
 		    b1_best = b1;
 		    dur_best = dur;

            Ibest = nsolutions; 		
         end
	    end;
	end;
    
    % Choose desired solution
    %
    if nsolutions > 0
        fprintf(1, '\n');
        fprintf(1,'Solution(s) exists!\n');
        for n=1:nsolutions
            fprintf(1, '%d) %s\n', n, infoall{n});
        end
        if nsolutions > 1
            Isolution = input('Which pulse would you like to use? (leave empty for shortest pulse) ');
            if isempty(Isolution) || Isolution < 1 || Isolution > nsolutions
                Isolution = Ibest;
            end
            fprintf(1, 'Returning %s\n', infoall{Isolution});
        else
            Isolution = 1;
        end
        rf = rfall{Isolution};
        g = gall{Isolution};
        fs_best = fsall(Isolution);
    end
    
    end;
      
    
	
    if isempty(rf)
        fprintf(1,'No solution found! Trying to increase band ripples to determine limiting\n frequency specifications...\n')
    
        orig_min_order = SS_MIN_ORDER;
        SS_MIN_ORDER = 0;
      	fs_top = fs_bands(end);
        fs_best = fs_test(fs_top);

        d_max = 1*ones(size(d));
        d_min = d;
        tol_factor = 4;
        
        for Id = 1:length(d)
            % trying bisection search with increased ripple in each band to
            % determine which portion of frequency spec is limiting
            d_test = d;
            
            while (d_max(Id) - d_min(Id)) > d(Id)/tol_factor
                d_test(Id) = (d_max(Id) + d_min(Id))/2;
                
                if strfind(ss_type, 'Flyback')
                    [rf, g] = ss_flyback(ang, z_thk, z_tb, z_d, f, a, d_test, fs_best, ...
                        ptype, z_ftype, s_ftype, ss_type, ...
                        f_off, dbg);
                else
                    [rf, g] = ss_ep(ang, z_thk, z_tb, z_d, f, a, d_test, fs_best, ...
                        ptype, z_ftype, s_ftype, ss_type, ...
                        f_off, dbg);
                end;
                
                % update min/max ripple values
                if ~isempty(rf),
                    d_max(Id) = d_test(Id);
                else
                    d_min(Id) = d_test(Id);
                end;
            end
        end
        
        % find ripple values that create solutions
        Ifix = find(d_max < 1);
        if ~isempty(Ifix)
            [tempmin Imin] = min(d_max - d);
            d_fix = d; d_fix(Imin) = d_max(Imin);
            fprintf(1,'Solution found by increasing ripples in bands %s\n', int2str(Ifix));
            fprintf(1,'Returning pulse with increased ripple in band %d (%.1f to %.1f Hz):\n', Imin, f(2*Imin-1), f(2*Imin));
           
            if strfind(ss_type, 'Flyback')
                [rf, g] = ss_flyback(ang, z_thk, z_tb, z_d, f, a, d_fix, fs_best, ...
                    ptype, z_ftype, s_ftype, ss_type, ...
                    f_off, dbg);
            else
                [rf, g] = ss_ep(ang, z_thk, z_tb, z_d, f, a, d_fix, fs_best, ...
                    ptype, z_ftype, s_ftype, ss_type, ...
                    f_off, dbg);
            end;
            	    
            b1_best = max(abs(rf));
            dur_best = length(rf) * SS_TS;
            pow_best = sum(abs(rf).^2) * SS_TS;
         	fprintf(1, 'Fs: %6.1f B1: %5.3fG Power: %5.3e G^2 ms Dur: %4.1fms\n', fs_best, b1_best, pow_best, dur_best*1e3); 
           
            fprintf(1,'\nPulse specs should be modified by reducing bandwidths or increasing ripple in bands %s\n', int2str(Ifix));
            fprintf(1,'Increasing the max pulse duration or slice thickness may also help\n');

        
        else
            error('No solution found! Try reducing bandwidths, increasing ripple, increasing max duration, increasing slice thickness...')
        end
        
        
        SS_MIN_ORDER = orig_min_order;
    end;
    
    % Test RF
    %
    fmid = (f(1:2:end) + f(2:2:end))/2;
    bw = [-fs_best/2 fs_best/2];
    [f_plot,z_plot,m_plot] = ss_plot(g,rf,SS_TS, ptype,z_thk*2,bw,...
        SS_GAMMA, fmid);
    


   