%
%
% SS_GLOBALS
%
% Common global definitions for SS package
%
% Parameters
%   NAME             DESCRIPTION                                                    DEFAULT
%   ----------       -------------------------------------------------------        ---------
%   'Nucleus'        Nuclei of interest                                             'Hydrogen'
%   'Max Grad'       Maximum Gradient Strength (G/cm)                               4
%   'Max Slew'       Maximum Slew Rate (G/cm/ms)                                    15
%   'Sample Time'    Sampling Time (s)                                              4e-6
%   'Max B1'         Maximum B1 amplitude (G)                                       0.2
%   'Max Duration'   Maximum Pulse Duration (s)                                     20e-3
%   'Num Lobe Iters' Spectral Sampling Frequency Iterations                         10
%   'Equal Lobes'    Force Equal positive and negative gradient lobes               0
%   'Verse Fraction' Fraction of ramps to use with VERSE                            0.8
%   'Num Fs Test'    Spectral aliasing frequencys to test                           100
%   'Spect Correct'  Spectral Correction with actual sampling                       0
%   'SLR'            Shinnar-Le Roux Correction for large tip pulses                0
%   'Min Order'      Find minimum order FIR filter                                  1
%   'B1 Verse'       Apply B1-VERSE'ing for reduced peak power                      0
%   'Slew Penalty'   In B1-VERSE, use slew-rate penalty to reduce delay sensitivity 0



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Spectral-Spatial RF Pulse Design for MRI and MRSI MATLAB Package
%
% Authors: Adam B. Kerr and Peder E. Z. Larson
%
% (c)2007-2011 Board of Trustees, Leland Stanford Junior University and
%	The Regents of the University of California. 
% All Rights Reserved.
%
% Please see the Copyright_Information and README files included with this
% package.  All works derived from this package must be properly cited.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% $Header: /home/adam/cvsroot/src/ss/ss_globals.m,v 1.11 2012/02/01 00:41:23 peder Exp $
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% $Log: ss_globals.m,v $
% Revision 1.11  2012/02/01 00:41:23  peder
% Added standard copyright notice to all functions
%
% Revision 1.10  2011/09/27 00:50:29  peder
% added documentation to ss_globals, working on demo.m
%
% Revision 1.9  2007-10-31 19:49:13  peder
% Added slew rate penalty in B1 verse'ing that reduces the allowable slew rate as the RF amplitude increases.
% Implemented in this package as a new option and global variable.
% Input to ss_b1verse changed.
%
% Revision 1.8  2007-10-25 21:06:29  adam
% Add option for not searching for minimum order pulse
%
% Revision 1.7  2007-10-05 16:45:03  adam
% Add SLR flag, set default SS_SPECT_CORRECT to 0
%
% Revision 1.6  2007-09-24 21:51:59  peder
% Updated for max B1 verse'ing.  Currently only supported in flyback design, need to add for ep design
%
% Revision 1.5  2007-09-07 17:26:47  adam
% Integrating ss_spect_correct, as well as EP support, almost(!!) working
%
% Revision 1.4  2007-09-05 20:59:25  adam
% Add water/fat examples
% First pass at adding symmetric filter design---still needs some work
% Get rid of SLR trial in EPI spectral correct routine---will fix later
%
% Revision 1.3  2007-08-31 00:00:50  adam
% Adding B1-minimization loop
%
% Revision 1.2  2007-08-30 00:23:58  adam
% Adding VERSE_FRACTION capability
%
% Revision 1.1.1.1  2007-08-29 15:58:20  adam
% Starting sources
%
% Revision 1.1.1.1  2007-08-29 15:45:32  adam
% Starting sources
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Gamma definitions -- all in Hz/G
%
SS_GAMMA_HYDROGEN = 4257.6;  
SS_GAMMA_LITHIUM = 1654.6;
SS_GAMMA_CARBON = 1070.5;
SS_GAMMA_SODIUM = 1126.2;
SS_GAMMA_PHOSPHOROUS = 1723.5;

% Declare globals
%
global SS_INIT_DONE; 

% Nucleus info
%
global SS_NUCLEUS SS_GAMMA; 

% Gradient/timing parameters
%
global SS_MXG SS_MXS SS_TS;

% RF parameters
%
global SS_MAX_B1 SS_MAX_DURATION;

% Design tolerances, parameters
%
global SS_NUM_LOBE_ITERS SS_EQUAL_LOBES SS_VERSE_FRAC SS_NUM_FS_TEST;
global SS_SPECT_CORRECT_FLAG SS_SLR_FLAG;
global SS_VERSE_B1 SS_SLEW_PENALTY SS_MIN_ORDER;

% Define globals if not already defined
%
if isempty(SS_INIT_DONE), 
    % Indicate init has been done
    %
    SS_INIT_DONE = 1;
    
    % Nucleus info
    %
    SS_NUCLEUS = 'Hydrogen';
    SS_GAMMA = SS_GAMMA_HYDROGEN;

    % Gradient/timing parameters
    %
    SS_MXG = 4.0;			% G/cm
    SS_MXS = 15;			% G/cm/ms 
    SS_TS = 4e-6;			% Sampling time (s) 

    % RF parameters
    %
    SS_MAX_B1 = 0.2; 			% Gauss
    SS_MAX_DURATION = 20e-3;		% Max allowed duration

    
    % Design tolerances, parameters
    %
    SS_NUM_LOBE_ITERS = 10;
    SS_EQUAL_LOBES = 0;
    SS_VERSE_FRAC = 0.8;
    SS_NUM_FS_TEST = 100;
    SS_SPECT_CORRECT_FLAG = 0;
    SS_SLR_FLAG = 0;
    SS_MIN_ORDER = 1;
    SS_VERSE_B1 = 0;
    SS_SLEW_PENALTY = 0;
end;
