function options = ss_opt(new_options)
% SS_OPT - Set options for spectral-spatial design
%   
% function options = ss_opt(new_options)
%
% options - cell array of current options
% new_options - cell array of new options to set
%
% Pass new_options as [] to reset options
%
% See help ss_globals for options

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Spectral-Spatial RF Pulse Design for MRI and MRSI MATLAB Package
%
% Authors: Adam B. Kerr and Peder E. Z. Larson
%
% (c)2007-2011 Board of Trustees, Leland Stanford Junior University and
%	The Regents of the University of California. 
% All Rights Reserved.
%
% Please see the Copyright_Information and README files included with this
% package.  All works derived from this package must be properly cited.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% $Header: /home/adam/cvsroot/src/ss/ss_opt.m,v 1.12 2012/02/01 00:41:23 peder Exp $
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% $Log: ss_opt.m,v $
% Revision 1.12  2012/02/01 00:41:23  peder
% Added standard copyright notice to all functions
%
% Revision 1.11  2011/09/27 00:50:29  peder
% added documentation to ss_globals, working on demo.m
%
% Revision 1.10  2007-10-31 19:49:13  peder
% Added slew rate penalty in B1 verse'ing that reduces the allowable slew rate as the RF amplitude increases.
% Implemented in this package as a new option and global variable.
% Input to ss_b1verse changed.
%
% Revision 1.9  2007-10-25 21:06:29  adam
% Add option for not searching for minimum order pulse
%
% Revision 1.8  2007-10-05 16:45:37  adam
% Add support for SLR setting
%
% Revision 1.7  2007-09-24 21:51:59  peder
% Updated for max B1 verse'ing.  Currently only supported in flyback design, need to add for ep design
%
% Revision 1.6  2007-09-07 17:26:47  adam
% Integrating ss_spect_correct, as well as EP support, almost(!!) working
%
% Revision 1.5  2007-09-05 20:59:25  adam
% Add water/fat examples
% First pass at adding symmetric filter design---still needs some work
% Get rid of SLR trial in EPI spectral correct routine---will fix later
%
% Revision 1.4  2007-08-31 00:00:50  adam
% Adding B1-minimization loop
%
% Revision 1.3  2007-08-30 05:21:18  adam
% Fixed up some nuisance bugs.
% Saw some strange behaviour with 'verse' and so modified it slightly
% into ss_verse.  Still not sure it is best it can be.
%
% Revision 1.2  2007-08-30 00:23:58  adam
% Adding VERSE_FRACTION capability
%
% Revision 1.1.1.1  2007-08-29 15:58:20  adam
% Starting sources
%
% Revision 1.1.1.1  2007-08-29 15:45:32  adam
% Starting sources
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    ss_globals;

    if (nargin < 1)
	error('Usage: options = ss_opt(new_options)');
    end;
    
    if isempty(new_options)
	SS_INIT_DONE = [];
	ss_globals;
    end;

    new_options = new_options.';
    new_options = new_options(:);
    if (mod(length(new_options), 2) == 1), 
	error('Input parameter (new_options) must be of even length');
    end;

    nopt = length(new_options)/2;
    for idx = 1:nopt, 
	opt = new_options{idx*2-1};
	optparm = new_options{idx*2};
	switch opt,
	 
	 case 'Nucleus'			% Set nucleus and gamma
	  switch optparm,
	   case 'Hydrogen'
	    SS_NUCLEUS = optparm;
	    SS_GAMMA = SS_GAMMA_HYDROGEN;
	   case 'Lithium'
	    SS_NUCLEUS = optparm;;
	    SS_GAMMA = SS_GAMMA_LITHIUM;
	   case 'Carbon'
	    SS_NUCLEUS = optparm;
	    SS_GAMMA = SS_GAMMA_CARBON;
	   case 'Sodium'
	    SS_NUCLEUS = optparm;
	    SS_GAMMA = SS_GAMMA_SODIUM;
	   case 'Phosphorous'
	    SS_NUCLEUS = optparm;
	    SS_GAMMA = SS_GAMMA_PHOSPHOROUS;
	   otherwise
	    error(sprintf('Nucleus: %s not recognized', optparm));
	  end;
	 
	 case 'Max Grad',
	  SS_MXG = optparm;
	 
	 case 'Max Slew', 
	  SS_MXS = optparm;
	  
	 case 'Max B1', 
	  SS_MAX_B1 = optparm;
	  
	 case 'Max Duration', 
	  SS_MAX_DURATION = optparm;
	  
	 case 'Num Lobe Iters', 
	  SS_NUM_LOBE_ITERS = optparm;
	  
	 case 'Equal Lobes', 
	  SS_EQUAL_LOBES = optparm;
	  
	 case 'Verse Fraction', 
	  SS_VERSE_FRAC = optparm;
	  
	 case 'Num Fs Test', 
	  SS_NUM_FS_TEST = optparm;
	  
	 case 'Spect Correct', 
	  SS_SPECT_CORRECT_FLAG = optparm;
	  
	 case 'SLR', 
	  SS_SLR_FLAG = optparm;
	  
	 case 'Min Order', 
	  SS_MIN_ORDER = optparm;
	  
	 case 'B1 Verse', 
	  SS_VERSE_B1 = optparm;
	  
	 case 'Slew Penalty', 
	  SS_SLEW_PENALTY = optparm;

     otherwise
	  SS_INIT_DONE = [];
	  ss_globals;
	  error(sprintf('Option: %s not recognized',opt));
	end;
    end;

    % Fill in options to pass back
    %
    idx = 1;

    options{idx, 1} = 'Nucleus'; 
    options{idx, 2} = SS_NUCLEUS; 
    idx = idx+1;
    
    options{idx, 1} = 'Max Grad';
    options{idx, 2} = SS_MXG;
    idx = idx+1;

    options{idx, 1} = 'Max Slew';
    options{idx, 2} = SS_MXS;
    idx = idx+1;

    options{idx, 1} = 'Sample Time'; 
    options{idx, 2} = SS_TS; 
    idx = idx+1;

    options{idx, 1} = 'Max B1'; 
    options{idx, 2} = SS_MAX_B1; 
    idx = idx+1;

    options{idx, 1} = 'Max Duration'; 
    options{idx, 2} = SS_MAX_DURATION; 
    idx = idx+1;

    options{idx, 1} = 'Num Lobe Iters'; 
    options{idx, 2} = SS_NUM_LOBE_ITERS; 
    idx = idx+1;

    options{idx, 1} = 'Equal Lobes'; 
    options{idx, 2} = SS_EQUAL_LOBES; 
    idx = idx+1;

    options{idx, 1} = 'Verse Fraction'; 
    options{idx, 2} = SS_VERSE_FRAC; 
    idx = idx+1;

    options{idx, 1} = 'Num Fs Test'; 
    options{idx, 2} = SS_NUM_FS_TEST; 
    idx = idx+1;

    options{idx, 1} = 'Spect Correct'; 
    options{idx, 2} = SS_SPECT_CORRECT_FLAG; 
    idx = idx+1;

    options{idx, 1} = 'SLR'; 
    options{idx, 2} = SS_SLR_FLAG; 
    idx = idx+1;

    options{idx, 1} = 'Min Order'; 
    options{idx, 2} = SS_MIN_ORDER; 
    idx = idx+1;

    options{idx, 1} = 'B1 Verse'; 
    options{idx, 2} = SS_VERSE_B1; 
    idx = idx+1;

    options{idx, 1} = 'Slew Penalty'; 
    options{idx, 2} = SS_SLEW_PENALTY; 
    idx = idx+1;


